<!doctype html>
<html lang="en">
  <head>
    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width, initial-scale=1, shrink-to-fit=no">

    <title>API</title>

    <!-- Bootstrap core CSS -->
    <link href="{{ '/static/css/bootstrap.min.css' }}" rel="stylesheet">

  </head>

  <body>
    {% block content %}{% endblock %}
  </body>
</html>
