{% extends "base.html" %}

{% block content %}
<main role="main">

  <!-- Main jumbotron for a primary marketing message or call to action -->
  <div class="jumbotron">
    <div class="container">
      <h1 class="display-3">API Index</h1>
      <p>TBD: Generating index.</p>
    </div>
  </div>

</main>
{% endblock %}
