import git
#git.Git("/your/directory/to/clone").clone("git://gitorious.org/git-python/mainline.git")

index = [
    'https://gitlab.com/wefindx/infinity.git',
    'https://gitlab.com/wefindx/metaculus.git',
    'https://gitlab.com/wefindx/hthworldwide.git',
    'https://gitlab.com/wefindx/kompass.git',
    'https://gitlab.com/wefindx/linkedin.git',
    'https://gitlab.com/wefindx/halfbakery.git',
    'https://gitlab.com/wefindx/metaculus.git',
    # 'https://gitlab.com/wefindx/3d-systems-prox-dmp-300.git',
    # 'https://gitlab.com/wefindx/biodigitalapp.git',
    # 'https://gitlab.com/wefindx/cdcvitals.git',
    # 'https://gitlab.com/wefindx/flightradar24.git',
    # 'https://gitlab.com/wefindx/google.git',
    # 'https://gitlab.com/wefindx/hp-metal-jet.git',
    # 'https://gitlab.com/wefindx/hscodeseu.git',
    # 'https://gitlab.com/wefindx/huodongxing.git',
    # 'https://gitlab.com/wefindx/infinity.git',
    # 'https://gitlab.com/wefindx/jianshu.git',
    # 'https://gitlab.com/wefindx/kakotalk.git',
    # 'https://gitlab.com/wefindx/kik.git',
    # 'https://gitlab.com/wefindx/kr36.git',
    # 'https://gitlab.com/wefindx/lietuvai.git',
    # 'https://gitlab.com/wefindx/lineapp.git',
    # 'https://gitlab.com/wefindx/marinetraffic.git',
    # 'https://gitlab.com/wefindx/meetupapp.git',
    # 'https://gitlab.com/wefindx/opencorporates.git',
    # 'https://gitlab.com/wefindx/quora.git',
    # 'https://gitlab.com/wefindx/reddit.git',
    # 'https://gitlab.com/wefindx/skyscanner.git',
    # 'https://gitlab.com/wefindx/telegram.git',
    # 'https://gitlab.com/wefindx/treebase.git',
    # 'https://gitlab.com/wefindx/twitter.git',
    # 'https://gitlab.com/wefindx/ubio.git',
    # 'https://gitlab.com/wefindx/vimeo.git',
    # 'https://gitlab.com/wefindx/wechat.git',
    # 'https://gitlab.com/wefindx/weibo.git',
    # 'https://gitlab.com/wefindx/whatsapp.git',
    # 'https://gitlab.com/wefindx/windy.git',
    # 'https://gitlab.com/wefindx/youku.git',
    # 'https://gitlab.com/wefindx/youtube.git',
    # 'https://gitlab.com/wefindx/zhihu.git',
]
