.mosaic.mosaic-thin-theme {
  position: relative;
  z-index: 0;
}

.mosaic.mosaic-thin-theme .mosaic-root {
  top: -2px;
  right: -2px;
  bottom: -2px;
  left: -2px;
}

.mosaic.mosaic-thin-theme .mosaic-tile {
  margin: 2px;
}

.mosaic.mosaic-thin-theme .mosaic-split.-row {
  margin-left: -2px;
  width: 4px;
}

.mosaic.mosaic-thin-theme .mosaic-split.-row .mosaic-split-line {
  left: 2px;
  right: 2px;
}

.mosaic.mosaic-thin-theme .mosaic-split.-column {
  margin-top: -2px;
  height: 4px;
}

.mosaic.mosaic-thin-theme .mosaic-split.-column .mosaic-split-line {
  top: 2px;
  bottom: 2px;
}

.mosaic.mosaic-thin-theme .mosaic-window,
.mosaic.mosaic-thin-theme .mosaic-window .mosaic-window-toolbar,
.mosaic.mosaic-thin-theme .mosaic-window .mosaic-window-body,
.mosaic.mosaic-thin-theme .mosaic-preview,
.mosaic.mosaic-thin-theme .mosaic-preview .mosaic-window-toolbar,
.mosaic.mosaic-thin-theme .mosaic-preview .mosaic-window-body {
  border-radius: 0;
}
