.contextmenu {
}

.contextmenu_error {
  padding: 10px;
  color: #8a9ba8;
  text-align: center;
}

.contextmenu_loading {
  display: flex;
  height: 100px;
  align-items: center;
  justify-content: center;
}

.contextmenu_subtitle {
  padding: 5px;
  min-width: 200px;
  color: #8a9ba8;
}

.contextmenu_subtitle::after {
  content: ':';
}

.drives {
  margin: 0;
  padding: 0;
  list-style-type: none;
}

.drives_item {
  border-bottom: 1px solid #bfccd6;
}

.drives_item:first-of-type {
  border-top: 1px solid #bfccd6;
}

.drive {
  margin: 0;
  display: flex;
  align-items: stretch;
}

.drive_radio {
  display: flex;
  width: 30px;
  flex-shrink: 0;
  flex-grow: 0;
  align-items: center;
  justify-content: center;
}

.drive_id {
  padding: 10px 10px 10px 0;
  display: flex;
  flex-grow: 1;
  min-height: 40px;
  align-items: center;
}

.drive_status {
  display: flex;
  width: 60px;
  flex-shrink: 0;
  flex-grow: 0;
  align-items: center;
  justify-content: center;
  color: #0f9960;
  border-left: 1px solid #bfccd6;
}

.drive.drive--dead {
}

.drive.drive--dead .drive_id {
  opacity: 0.3;
}

.drive.drive--dead .drive_status {
  color: #8a9ba8;
}
