.search {
  height: 100%;
  display: flex;
  flex-direction: column;
}

.search_navbar {
  flex-shrink: 0;
}

.search_results {
  flex-grow: 1;
  overflow-y: scroll;
}

.search_loading {
  display: flex;
  height: 100%;
  align-items: center;
  justify-content: center;
}

.search_updating {
  display: flex;
  height: 100px;
  align-items: center;
  justify-content: center;
}
